from pandiag import dot, markdown, mermaid
from pandiag.model import Graph, Subgraph, Edge

import argparse

FORMATTERS = {
    'dot': dot.format,
    'markdown': markdown.format,
    'mermaid': mermaid.format,
}

GRAPH = Graph(
    directed=True,
    rootgraph=Subgraph(
        edges=[
            Edge('a', 'b'),
            Edge('b', 'c'),
            Edge('b', 'd'),
        ]
    )
)

def main():
    parser = argparse.ArgumentParser(description='Formats a simple graph')
    parser.add_argument('-o', '--output', help='The output file path')
    parser.add_argument('-f', '--format', choices=sorted(FORMATTERS.keys()), default='dot')

    args = parser.parse_args()
    s = FORMATTERS[args.format](GRAPH)
    if args.output and args.output != '-':
        with open(args.output, 'w') as f:
            f.write(s)
    else:
        print(s)

if __name__ == '__main__':
    main()



import argparse
import sys

from dataclasses import dataclass, field
from pathlib import Path
from typing import Callable, Optional

from pandiag import dot, drawio, mermaid
from pandiag.model.graph import Graph

@dataclass
class Format:
    extensions: list[str] = field(default_factory=list)
    format: Optional[Callable[[Graph], str | bytes]] = None
    parse: Optional[Callable[[str], Graph]] = None

FORMATS = {
    'dot': Format(extensions=['dot'], format=dot.format),
    'drawio': Format(extensions=['drawio'], parse=drawio.parse),
    'markdown': Format(extensions=['md', 'markdown'], format=mermaid.format_markdown),
    'mermaid': Format(extensions=[], format=mermaid.format),
    'pdf': Format(extensions=['pdf'], format=dot.format_pdf),
    'png': Format(extensions=['png'], format=dot.format_png),
}

def guess_format(path: Path) -> Optional[Format]:
    extension = path.name.rsplit('.', maxsplit=1)[-1]
    return next((f for f in FORMATS.values() if extension in f.extensions), None)

def main():
    parser = argparse.ArgumentParser(description='Utility for converting between diagram formats')
    parser.add_argument('-o', '--output', type=Path, required=True, metavar='PATH', help='The path to the output document')
    parser.add_argument('input', type=Path, help='The path to the input document')

    args = parser.parse_args()
    
    if not (input_format := guess_format(args.input)):
        print(f'Unrecognized input format: {args.input.name}')
        sys.exit(1)
    
    if not (output_format := guess_format(args.output)):
        print(f'Unrecognized output format: {args.output.name}')
        sys.exit(1)

    if not input_format.parse:
        print('Input format does not support parsing')
        sys.exit(1)

    if not output_format.format:
        print('Output format does not support formatting')
        sys.exit(1)

    with open(args.input, 'r') as f:
        raw_input = f.read()

    graph = input_format.parse(raw_input)
    raw_output = output_format.format(graph)

    with open(args.output, 'w' if isinstance(raw_output, str) else 'wb') as f:
        f.write(raw_output)



from typing import Optional
from pandiag.model.graph import Edge, Graph, Subgraph
from pandiag.utils import indent

import hashlib
import re

def _format_node(label: Optional[str]) -> str:
    # TODO: Generate more readable node names than SHA1 hashes
    # TODO: Find a better way to sanitize the label
    label = re.sub(r'[()]', '', label).strip() if label else None
    return f"{hashlib.sha1(label.encode()).hexdigest()}[{label}]" if label else 'None'

def _format_edge(edge: Edge, graph: Graph) -> str:
    return f"{_format_node(edge.source)} {'-->' if graph.directed else '---'} {_format_node(edge.dest)}"

def _format_subgraph(subgraph: Subgraph, graph: Graph) -> list[str]:
    return [
        *(_format_edge(e, graph=graph) for e in subgraph.edges),
        *(l for g in subgraph.subgraphs for l in [
            f'subgraph {_format_node(g.name)}',
            *indent(_format_subgraph(g, graph)),
            'end',
        ]),
    ]

def _format_graph(graph: Graph) -> list[str]:
    return [
        'flowchart TD',
        *indent(_format_subgraph(graph.rootgraph, graph=graph)),
    ]

def format(graph: Graph) -> str:
    return '\n'.join(_format_graph(graph))

def format_markdown(graph: Graph) -> str:
    return '\n'.join([
        '```mermaid',
        format(graph),
        '```',
    ])



from typing import Optional
from pandiag.model.graph import Edge, Graph, Subgraph
from pandiag.utils import indent

import subprocess

def _format_node(label: Optional[str]) -> str:
    return f'"{label.strip()}"' if label else None

def _format_edge(edge: Edge, graph: Graph) -> str:
    return f"{_format_node(edge.source)} {'->' if graph.directed else '--'} {_format_node(edge.dest)};"

def _format_subgraph(subgraph: Subgraph, graph: Graph) -> list[str]:
    return [
        *(_format_edge(e, graph=graph) for e in subgraph.edges),
        *(l for g in subgraph.subgraphs for l in [
            f"subgraph {_format_node(f'cluster_{g.name}')} {{",
            *indent([
                *([f'label = {_format_node(g.name)};'] if g.name else ''),
                *_format_subgraph(g, graph),
            ]),
            '}',
        ]),
    ]

def _format_graph(graph: Graph) -> list[str]:
    return [
        f"{'digraph' if graph.directed else 'graph'} {_format_node(graph.rootgraph.name)} {{",
        *indent(_format_subgraph(graph.rootgraph, graph=graph)),
        '}',
    ]

def format(graph: Graph) -> str:
    return '\n'.join(_format_graph(graph))

def format_rendered(graph: Graph, render_format: str) -> bytes:
    result = subprocess.run(
        ['dot', '-T', render_format],
        input=format(graph).encode(),
        capture_output=True
    )
    return result.stdout

def format_png(graph: Graph) -> bytes:
    return format_rendered(graph, 'png')

def format_pdf(graph: Graph) -> bytes:
    return format_rendered(graph, 'pdf')
